/*------------------------------------------------*/
/* 26regexp/posix.c                               */

#include "parse-options.h"
#include <stdlib.h>
#include <regex.h>
#include <locale.h>

#define N 50

static void regerr(regex_t *regex, int ret);
static void printPatterns(size_t n, regmatch_t const pmatch[], char * text);


int main(int argc, char *argv[]) {
    regex_t regex;
    task t;
    int ret, cflags;
    regmatch_t pmatch[N];

    setlocale(LC_ALL,"");

    t = parseOptions(argc, argv);
    if(t.help || t.err) { 
        printOptionsHelp(); 
        return 1; 
    }
    if(t.show) {
        printOptions(t);
    }
    printf("\n");

    cflags = 0;
    if(t.extended)
        cflags |= REG_EXTENDED;
    if(t.icase) 
        cflags |= REG_ICASE;
    if(t.newline)
        cflags |= REG_NEWLINE;

    /* Kompilace reg. výrazu */
    ret = regcomp(&regex, t.pattern, cflags);
    if(ret) {
        fprintf(stderr,"Nepodařilo se zkompilovat reg. exp. '%s'\n\n", t.pattern);
        regerr(&regex, ret);
        return 1;
    }

    /* Vyhledávání reg. výrazů */
    ret = regexec(&regex, t.string, sizeof(pmatch)/sizeof(regmatch_t), pmatch, 0);
    if (!ret) {
        puts("PATTERN BYL NALEZEN.");
	/* vypsání nalezených částí reg. výrazu */
        printPatterns(N,pmatch, t.string);
    }
    else if (ret == REG_NOMATCH) {
        puts("PATTERN nebyl NALEZEN.");
    }
    else {
        puts("CHYBA");
        regerr(&regex, ret);
        return 1;
    }
    regfree(&regex);
    printf("\n");

    return 0;
}

/**
 * Vypíše chybu
 */
static void regerr(regex_t *regex, int ret) {
    char msgbuf[100];
    regerror(ret, regex, msgbuf, sizeof(msgbuf));
    fprintf(stderr, "Regex match failed: %s\n\n", msgbuf);
}

/**
 * Vypíše nalezené části textu
 */
static void printPatterns(size_t n, regmatch_t const pmatch[], char * text) {
    size_t i, last = 0;
    int length;
    /* hledam posledni nalezeny text */
    for(i = 0; i < n; i++) {
        if(pmatch[i].rm_so != -1) last = i;
    }
    /* vypisuji od zacatku az k poslednimu nalezenemu textu */
    for(i = 0; i <= last; i++) {
        length = pmatch[i].rm_eo - pmatch[i].rm_so;
        printf("(%i, %i) %i '%.*s'\n",pmatch[i].rm_so, pmatch[i].rm_eo, length, length, text+pmatch[i].rm_so);
    }
}

/*------------------------------------------------*/
