/*------------------------------------------------*/
/* 27aDom/config.c                                */

#include <stdlib.h>
#include <stdio.h>
#include "config.h"

/**
* Vytvoří strukturu Config. Všechny položky 
* jsou inicializovány na 0,NULL,false nebo prázdné řetězce.
* @param size_t databases Počet, pro kolik databází se alokuje paměť
* @return Config Vrací NULL, pokud se nepodaří alokovat paměť
*/
Config *createConfigStructure(size_t databases)
{
    Config *config;
    config = (Config *) calloc(1, sizeof(Config));
    if (!config) {
        return NULL;
    }
    config->databases = (Database *) calloc(databases, sizeof(Database));
    if (!config->databases) {
        free(config);
        return NULL;
    }
    config->databasesSize = databases;
    return config;
}

/**
* Uvolní paměť alokovanou pro config
* @param Config * config
*/
void freeConfigStructure(Config * config)
{
    if (!config)
        return;
    free(config->databases);
    free(config);
}

int getDefaultPort(enum databaseType type)
{
    switch (type) {
    case mysql:
        return 3306;
    case postgres:
        return 5432;
    case oracle:
        return 1521;
    case sqlite:
        return 0;
    default:
        return 0;
    }
}

/**
* @param char * Textový název databáze dle enum typu
* @return enum DatabaseType 
*/
enum databaseType getDatabaseTypeFromText(char *type)
{
    if (!strcmp(type, "mysql"))
        return mysql;
    if (!strcmp(type, "postgres"))
        return postgres;
    if (!strcmp(type, "oracle"))
        return oracle;
    if (!strcmp(type, "sqlite"))
        return sqlite;
    return unknown;
}

/**
* Pomocná funkce pro funkci printconfigStructure()
* @param enum DatabaseType type
* @return char * Textový název databáze dle enum typu
*/
static char *getDatabaseTypeAsText(enum databaseType type)
{
    switch (type) {
    case mysql:
        return "MySQL";
    case postgres:
        return "PostgreSQL";
    case oracle:
        return "Oracle";
    case sqlite:
        return "SQLite 2";
    default:
        return "unknown";
    }
}

/**
* Vytiskne strukturu Config
* @param Config * config
*/
void printConfigStructure(Config * config)
{
    size_t i;
    Database *d;
    printf("Version: %s.%s\n", config->version.major,
           config->version.minor);
    for (i = 0; i < config->databasesSize; i++) {
        d = &config->databases[i];
        printf("Database: %s; %s\n",
               getDatabaseTypeAsText(d->type),
               d->enabled ? "enabled" : "");
        printf("\tUsername: %s\n" "\tPassword: %s\n" "\tSchema: %s\n"
               "\tHost: %s\n" "\tPort: %i\n", d->name, d->password,
               d->schema, d->host, d->port);
        printf("\n");
    }
}

/*------------------------------------------------*/
