/*------------------------------------------------*/
/* 27aDom/dom-parser.c                            */

#include <stdbool.h>
#include <string.h>
#include "dom-parser.h"
#include "dom-utils.h"

static bool parseVersion(xmlDocPtr doc, xmlNodePtr cur, Config * config);
static int parseDatabases(xmlDocPtr doc, xmlNodePtr cur, Config * config);
static bool parseDatabase(xmlDocPtr doc, xmlNodePtr cur, Database * database);

void parseConfigDocument(xmlDocPtr doc, xmlNodePtr cur, Config * config)
{
    cur = cur->xmlChildrenNode;
    while (cur != NULL) {
        if ((!xmlStrcmp(cur->name, (const xmlChar *)"version"))) {
            parseVersion(doc, cur, config);
        }
        if ((!xmlStrcmp(cur->name, (const xmlChar *)"databases"))) {
            parseDatabases(doc, cur, config);
        }
        cur = cur->next;
    }
}

static bool parseVersion(xmlDocPtr doc, xmlNodePtr cur, Config * config)
{
    bool nalezeno[2] = { false, false };
    cur = cur->xmlChildrenNode;
    while (cur != NULL) {
        if ((!xmlStrcmp(cur->name, (const xmlChar *)"major"))) {
            nalezeno[0] = true;
            copyXMLString(doc, cur, config->version.major,
                      sizeof(config->version.major) /
                      sizeof(config->version.major[0]));
        }
        if ((!xmlStrcmp(cur->name, (const xmlChar *)"minor"))) {
            nalezeno[1] = true;
            copyXMLString(doc, cur, config->version.minor,
                      sizeof(config->version.major) /
                      sizeof(char));
        }
        cur = cur->next;
    }
    return nalezeno[0] && nalezeno[1];
}

static int parseDatabases(xmlDocPtr doc, xmlNodePtr cur, Config * config)
{
    size_t i = 0;
    cur = cur->xmlChildrenNode;

    while ((i < config->databasesSize) && cur) {
        if ((!xmlStrcmp(cur->name, (const xmlChar *)"database"))) {
            parseDatabase(doc, cur, &(config->databases[i]));
            i++;
        }
        cur = cur->next;
    }
    return i;
}

static bool parseDatabase(xmlDocPtr doc, xmlNodePtr cur, Database * database)
{
    bool nalezeno[] = { false, false, false, false, false, false };
    bool ntrues[] = { true, true, true, true, true, true };
    char tmp[10];

    if (getProperty(cur, "type", tmp, sizeof(tmp))) {
        nalezeno[0] = true;
        database->type = getDatabaseTypeFromText(tmp);
    }

    if (getProperty(cur, "enabled", tmp, sizeof(tmp))) {
        nalezeno[1] = true;
        database->enabled = !strcmp(tmp, "true");
    }

    cur = cur->xmlChildrenNode;
    while (cur != NULL) {
        if ((!xmlStrcmp(cur->name, (const xmlChar *)"name"))) {
            nalezeno[2] = true;
            copyXMLString(doc, cur, database->name,
                      sizeof(database->name));
        } else if ((!xmlStrcmp(cur->name, (const xmlChar *)"password"))) {
            nalezeno[3] = true;
            copyXMLString(doc, cur, database->password,
                      sizeof(database->password));
        }
        if ((!xmlStrcmp(cur->name, (const xmlChar *)"schema"))) {
            nalezeno[4] = true;
            copyXMLString(doc, cur, database->schema,
                      sizeof(database->schema));
        }
        if ((!xmlStrcmp(cur->name, (const xmlChar *)"host"))) {
            nalezeno[5] = true;
            copyXMLString(doc, cur, database->host,
                      sizeof(database->host));
        }
        if ((!xmlStrcmp(cur->name, (const xmlChar *)"port"))) {
            copyXMLInteger(doc, cur, &(database->port));
        }
        cur = cur->next;
    }
    if (!database->port) {
        database->port = getDefaultPort(database->type);
    }
    return !memcmp(nalezeno, ntrues,
               sizeof(nalezeno) / (sizeof(nalezeno[0])));
}

/*------------------------------------------------*/
