/*------------------------------------------------*/
/* 51ncurses/threads.c                            */
#include <stdio.h>
#include <locale.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <pthread.h>
#include "thread-ncurses.h"

static void obnovTerminal()
{
    printw("Obnovuji nastavení terminálu\n");
    thncurses_stop();
    endwin();
    printf("Terminal obnoven\n");
}

static void initNcurses() {
    setlocale(LC_ALL, "");
    initscr();
    noecho();
    raw();
    atexit(obnovTerminal);

    if (!has_colors()) {
        fprintf(stderr, "Terminál nepodporuje barvy\n");
        exit(EXIT_FAILURE);
    }
    if (start_color() != OK) {
        fprintf(stderr, "Nepodařilo se inicializovat barvy\n");
        exit(EXIT_FAILURE);
    }
}

#define O_RADKU 5
#define O_SLOUPCU 50

typedef struct {
    int lines;
    int cols;
    int line;
    int col;
} wPosition;

static WINDOW *okno = NULL;
static WINDOW *subokno = NULL;
static int refreshed = 0;
static int pair_okno = 1;
static int col = 0;
static int line = 0;
static wPosition p;

static wPosition getWindowPosition()
{
    p.lines = LINES / 2;
    p.cols = COLS / 2;
    line = p.line = LINES / 2 - LINES / 4;
    col = p.col = COLS / 2 - COLS / 4;
    return p;
}

static void prekresliOkno()
{
    int i, j;
    wPosition p = getWindowPosition();
    wattrset(okno, COLOR_PAIR(pair_okno));
    wattrset(subokno, COLOR_PAIR(pair_okno));
    for (i = 0; i < p.lines; i++)
        for (j = 0; j < p.cols; j++)
            mvwprintw(okno, i, j, "%c", ' ');
    box(okno, ACS_VLINE, ACS_HLINE);
    if(p.lines/2 >= 2)
    mvwprintw(subokno, p.lines / 2 - 1, 0,
          "LINES = %i, COLS = %i, refresed %i times.", LINES, COLS,
          refreshed);
}


static void zpracuj_winch_job(void *arg)
{
    refreshed++;
    endwin();
    refresh();
    erase();
    refresh();
    wPosition p = getWindowPosition();
    if(p.lines > 2 && p.cols > 2) {
            wresize(okno, p.lines, p.cols);
            wresize(subokno, p.lines - 2, p.cols - 2);
    }
    mvwin(okno, p.line, p.col);
    prekresliOkno();
    touchwin(stdscr);
    touchwin(okno);
}

void zpracuj_winch() {
    thncurses_signal();
}

static void nastavSignal()
{
    struct sigaction sa;
    sigset_t set;
    sigemptyset(&set);
    sa.sa_handler = zpracuj_winch;
    sa.sa_mask = set;
    sa.sa_flags = SA_RESTART;
    sa.sa_restorer = NULL;
    sigaction(SIGWINCH, &sa, NULL);
}

static void *winch_thread(void *arg) {
    sigset_t set;
    sigemptyset(&set);
    sigaddset(&set, SIGWINCH);
    sigprocmask(SIG_UNBLOCK, &set, NULL);
    nastavSignal();
    while(true) {
        sleep(1000);
    }
    return NULL;
}

void show_title_job(void *i)
{
    mvwprintw(okno, 0, 0, "%s %i", "Vlákno ", *((int *)i));
}

static void *show_title_thread(void *arg)
{
    static int i = 0;
    THNcursesJob job;
    while (true) {
        job.fce = show_title_job;
        job.data = (void *)&i;
        thncurses_add_job(job);
        thncurses_paint();
        i++;
        //usleep(10);
    }
    return NULL;
}

void move_okno_job(void *position)
{
    wPosition *p = (wPosition *) position;
    mvwin(okno, p->line, p->col);
}

void clear_job(void *arg)
{
    touchwin(stdscr);
    wrefresh(stdscr);
}

int main(void)
{
    pthread_t title_t, thncurses_t, winch_t;
    THNcursesJob job;
    THNcursesJob tjob;

    initNcurses();
    curs_set(0);

    /* zablokuje SIGWINCH pro toto vlákno i všechna nově vytvořená vlánka */
    sigset_t set;
    sigemptyset(&set);
    sigaddset(&set, SIGWINCH);
    sigprocmask(SIG_BLOCK, &set, NULL);

    /* nastaveni barvy pozadi a prekresleni cele obrazovky */
    assume_default_colors(COLOR_WHITE, COLOR_MAGENTA);
    erase();
    refresh();

    getWindowPosition();
    okno = newwin(p.lines, p.cols, p.line, p.col);
    subokno = derwin(okno, p.lines - 2, p.cols - 2, 1, 1);

    thncurses_add_window(stdscr);
    thncurses_add_window(okno);
    init_pair(pair_okno, COLOR_RED, COLOR_WHITE);

    tjob.fce = zpracuj_winch_job;
    tjob.data = NULL;
    pthread_create(&thncurses_t, NULL, (void * (*)(void *)) thncurses_signal_thread, &tjob);
    pthread_detach(thncurses_t);
    zpracuj_winch_job(tjob.data);

    pthread_create(&winch_t, NULL, winch_thread, NULL);
    pthread_detach(winch_t);

    pthread_create(&title_t, NULL, show_title_thread, NULL);
    pthread_detach(title_t);

    keypad(stdscr, TRUE);
    int ch = '\0';
    wPosition np;
    do {
        thncurses_paint();
        ch = getch();
        switch (ch) {
        case KEY_LEFT:
            if (col > 0) {
                col--;
                np.line = line;
                np.col = col;
                job.fce = move_okno_job;
                job.data = (void *)&np;
                thncurses_add_job(job);
            }
            break;
        case KEY_RIGHT:
            if (col < COLS - p.cols) {
                col++;
                np.line = line;
                np.col = col;
                job.fce = move_okno_job;
                job.data = (void *)&np;
                thncurses_add_job(job);
            }
            break;
        case KEY_UP:
            if (line > 0) {
                line--;
                np.line = line;
                np.col = col;
                job.fce = move_okno_job;
                job.data = (void *)&np;
                thncurses_add_job(job);
            }
            break;
        case KEY_DOWN:
            if (line < LINES - p.lines) {
                line++;
                np.line = line;
                np.col = col;
                job.fce = move_okno_job;
                job.data = (void *)&np;
                thncurses_add_job(job);
            }
            break;
        case 'c':
            job.fce = clear_job;
            job.data = NULL;
            thncurses_add_job(job);
            thncurses_paint();
            break;
        }
    } while (ch && ch != '\n');
    return 0;
}

/*------------------------------------------------*/
